@@ -1,5 +1,8 @@
 import numpy as np
 import pandas as pd
+from synctoolbox.feature.pitch import __visualize_pitch
+from synctoolbox.feature.pitch_onset import __f_peaks_to_matrix
+import matplotlib.pyplot as plt
 
 
 def read_csv_to_df(csv_filepath: str = '',
@@ -37,7 +40,9 @@ def df_to_pitch_features(df: pd.DataFrame,
                          midi_max: int = 108,
                          transpose: int = 0,
                          ignore_velocity: bool = False,
-                         ignore_percussion: bool = False) -> np.ndarray:
+                         ignore_percussion: bool = False,
+                         visualize: bool = False,
+                         visualization_title: str = "Pitch features") -> np.ndarray:
     """ Computes pitch-based features for a dataframe containing symbolic music.
     The resulting features have the same format as the output of 'audio_to_pitch_features'
     for audio.
@@ -67,6 +72,12 @@ def df_to_pitch_features(df: pd.DataFrame,
     ignore_percussion : bool
         Ignores percussion. If `True`, no features are generated for percussive events.
 
+    visualize : bool
+        Set `True` to activate the visualization of features
+
+    visualization_title : str
+        Title for the visualization plot. Only relevant if 'visualize' is True
+
     Returns
     -------
     f_pitch : np.ndarray
@@ -93,9 +104,11 @@ def df_to_pitch_features(df: pd.DataFrame,
             raise ValueError(f'The pitch for note {pitch} at time point {start_time_ms/ 1000} sec is not valid.')
 
         # TODO: ATTENTION TO INDEXING!
-        if instrument == 'percussive' and not ignore_percussion:
-            for cur_win in range(first_window_involved, np.minimum(first_window_involved + 1, num_pitch_features) + 1):
-                f_pitch[pitch, cur_win-1] = __compute_pitch_energy(cur_energy_val=f_pitch[pitch, cur_win - 1],
+        if instrument == 'percussive':
+            # TODO The code for percussive events is not well tested!
+            if not ignore_percussion:
+                for cur_win in range(first_window_involved, np.minimum(first_window_involved + 1, num_pitch_features) + 1):
+                    f_pitch[:, cur_win-1] = __compute_pitch_energy(cur_energy_val=f_pitch[:, cur_win - 1],
                                                                    cur_win=cur_win,
                                                                    start_time_ms=start_time_ms,
                                                                    end_time_ms=end_time_ms,
@@ -113,6 +126,10 @@ def df_to_pitch_features(df: pd.DataFrame,
                                                                    stepsize_ms=stepsize_ms,
                                                                    ignore_velocity=ignore_velocity)
 
+    if visualize:
+        __visualize_pitch(f_pitch, feature_rate=feature_rate, plot_title=visualization_title)
+        plt.show()
+
     return f_pitch
 
 
@@ -121,7 +138,9 @@ def df_to_pitch_onset_features(df: pd.DataFrame,
                                midi_max: int = 108,
                                transpose: int = 0,
                                ignore_percussion: bool = False,
-                               peak_height_scale_factor: float = 1e6) -> dict:
+                               peak_height_scale_factor: float = 1e6,
+                               visualize: bool = False,
+                               visualization_title: str = "Pitch features") -> dict:
     """Computes pitch-based onset features for a dataframe containing symbolic music.
     The resulting features have the same format as the output of 'audio_to_pitch_onset_features'
     for audio.
@@ -149,14 +168,20 @@ def df_to_pitch_onset_features(df: pd.DataFrame,
         are in a similar range than the peak features from an audio file
         So 1e6 is more or less arbitrary.
 
+    visualize : bool
+        Set `True` to activate the visualization of features
+
+    visualization_title : str
+        Title for the visualization plot. Only relevant if 'visualize' is True
+
     Returns
     -------
     f_peaks : dict
         A dictionary of onset peaks, see 'audio_to_pitch_onset_features' for the exact format
     """
-    num_peaks_in_pitch = dict()
+    peak_height_scale_factor_percussion = peak_height_scale_factor * 10
+    num_peaks_in_pitch = {pitch: 0 for pitch in range(1, 129)}
     f_peaks = dict()
-    num_percussive_notes = 0
 
     for _, row in df.iterrows():
         start_time_ms = 1000 * row['start']
@@ -167,29 +192,28 @@ def df_to_pitch_onset_features(df: pd.DataFrame,
         if not midi_max >= pitch >= midi_min:
             raise ValueError(f'The pitch for note {pitch} at timepoint {start_time_ms/ 1000} sec is not valid.')
 
-        if instrument == 'percussive' and not ignore_percussion:
-            num_percussive_notes += 1
-            raise NotImplementedError()
-            # TODO
-            pass
+        def add_peak_for_pitch(p, scale_factor=peak_height_scale_factor):
+            num_peaks_in_pitch[p] += 1
 
+            if num_peaks_in_pitch[p] > 0:
+                if p not in f_peaks:
+                    f_peaks[p] = np.zeros((2, 1000))
+                if num_peaks_in_pitch[p] > f_peaks[p].shape[1]:
+                    f_peaks[p] = np.concatenate([f_peaks[p], np.zeros((2, 1000))], axis=1)
+
+            if p not in f_peaks or f_peaks[p].size == 0:
+                f_peaks[p] = np.array([[start_time_ms], [velocity / scale_factor]], np.float64)
+            else:
+                f_peaks[p][0, num_peaks_in_pitch[p] - 1] = start_time_ms
+                f_peaks[p][1, num_peaks_in_pitch[p] - 1] = velocity / scale_factor
+
+        if instrument == 'percussive':
+            # TODO The code for percussive events is not well tested!
+            if not ignore_percussion:
+                for p in range(1, 129):
+                    add_peak_for_pitch(p, peak_height_scale_factor_percussion)
         else:
-            if pitch not in num_peaks_in_pitch:
-                num_peaks_in_pitch[pitch] = 1
-            else:
-                num_peaks_in_pitch[pitch] += 1
-
-            if num_peaks_in_pitch[pitch] > 0:
-                if pitch not in f_peaks:
-                    f_peaks[pitch] = np.zeros((2, 1000))
-                if num_peaks_in_pitch[pitch] > f_peaks[pitch].shape[1]:
-                    f_peaks[pitch] = np.concatenate([f_peaks[pitch], np.zeros((2, 1000))], axis=1)
-
-            if pitch not in f_peaks or f_peaks[pitch].size == 0:
-                f_peaks[pitch] = np.array([[start_time_ms], [velocity / peak_height_scale_factor]], np.float64)
-            else:
-                f_peaks[pitch][0, num_peaks_in_pitch[pitch]-1] = start_time_ms
-                f_peaks[pitch][1, num_peaks_in_pitch[pitch]-1] = velocity / peak_height_scale_factor
+            add_peak_for_pitch(pitch)
 
     for pitch in f_peaks:
         time_vals = f_peaks[pitch][0, :][0: num_peaks_in_pitch[pitch]]
@@ -200,6 +224,12 @@ def df_to_pitch_onset_features(df: pd.DataFrame,
         f_peaks[pitch][0, :] = f_peaks[pitch][0, :][sort_index]
         f_peaks[pitch][1, :] = f_peaks[pitch][1, :][sort_index]
 
+    if visualize:
+        highest_time_res = 50 / 22050  # TODO don't hardcode this, identify appropriate resolution based on df
+        imagedata = __f_peaks_to_matrix(np.max(df["start"] + df["duration"]), f_peaks, highest_time_res, midi_max, midi_min)
+        __visualize_pitch(imagedata.T, midi_min, midi_max, feature_rate=1 / highest_time_res, plot_title=visualization_title)
+        plt.show()
+
     return f_peaks
 
 
@@ -252,20 +282,20 @@ def __compute_pitch_energy(cur_energy_val: float,
     contribution = (np.minimum(end_time_ms, right_border_cur_win_ms) -
                     np.maximum(start_time_ms, left_border_cur_win_ms)) / (2 * stepsize_ms)
 
-    # Add energy equally distributed to all pitches
-    # Since we assume the percussive sound to be short,
-    # we just add energy to the first relevant Window
     if is_percussive:
-        res = cur_energy_val + (velocity / 120) * contribution
-
-    # If not percussive,
-    # add energy for this note to features
-    # assume constant note energy throughout the whole note
-    #  this may later be improved to an ADSR model
-    elif ignore_velocity:
-        res = 1.0
+        # Add energy equally distributed to all pitches
+        # Since we assume the percussive sound to be short,
+        # we just add energy to the first relevant Window
+        res = cur_energy_val + (velocity / 128) * contribution
     else:
-        res = cur_energy_val + velocity * contribution
+        # If not percussive,
+        # add energy for this note to features
+        # assume constant note energy throughout the whole note
+        #  this may later be improved to an ADSR model
+        if ignore_velocity:
+            res = 1.0
+        else:
+            res = cur_energy_val + velocity * contribution
 
     return res
 
